use crate::{client::Client, result::Result};
use reqwest::{
    header::{HeaderMap, HeaderValue},
    Client as RClient,
};

use serde::Deserialize;

#[derive(Debug)]
pub struct GitLab {
    client: Client,
}

#[derive(Deserialize, Debug)]
pub struct User {
    pub id: u64,
    pub username: String,
    pub email: String,
    pub name: String,
    pub state: String,
    pub locked: bool,
    pub created_at: String,
    pub public_email: String,
}

impl GitLab {
    pub fn try_new(url: &str, token: &str) -> Result<Self> {
        let mut headers = HeaderMap::new();
        // TODO: ensure this token cannot be leaked to logs?
        headers.insert("PRIVATE-TOKEN", HeaderValue::from_str(token)?);
        headers.insert("content-type", HeaderValue::from_str("application/json")?);
        headers.insert("accepts", HeaderValue::from_str("application/json")?);

        let base_client = RClient::builder().default_headers(headers).build()?;

        let client = Client::try_new(base_client, url)?;

        Ok(Self { client })
    }

    pub async fn me(&self) -> Result<User> {
        self.client.get("/user").await
    }
}
